@@ -3,9 +3,9 @@
 ## author: Thomas Alexander Gerds
 ## created: Aug 15 2016 (09:45) 
 ## Version: 
-## last-updated: Feb  7 2020 (19:00) 
+## last-updated: Mar  9 2022 (15:49) 
 ##           By: Thomas Alexander Gerds
-##     Update #: 144
+##     Update #: 148
 #----------------------------------------------------------------------
 ## 
 ### Commentary: 
@@ -41,6 +41,7 @@
 ##' # binary outcome
 ##' library(data.table)
 ##' library(prodlim)
+##' set.seed(10)
 ##' db=sampleData(40,outcome="binary")
 ##' fitconv=glm(Y~X3+X5,data=db,family=binomial)
 ##' fitnew=glm(Y~X1+X3+X5+X6+X7,data=db,family=binomial)
@@ -122,7 +123,7 @@ boxplot.Score <- function(x,
 
     if (type=="diff"){
         warning("ATTENTION PLEASE: Boxplots of differences of predicted risk are non-proper\nin the sense that a miscalibrated model can apparently look good.")
-        pframe <- x$riskQuantile$contrasts[model!="Null model"]
+        pframe <- x$riskQuantile$contrasts[reference!="Null model"& model!="Null model"]
     } else{
         pframe <- x$riskQuantile$score[model!="Null model"]
     }
