@@ -3,9 +3,9 @@
 ## Author: Thomas Alexander Gerds
 ## Created: Nov  4 2019 (09:07) 
 ## Version: 
-## Last-Updated: Nov  4 2019 (12:21) 
+## Last-Updated: Jun 25 2020 (13:42) 
 ##           By: Thomas Alexander Gerds
-##     Update #: 12
+##     Update #: 20
 #----------------------------------------------------------------------
 ## 
 ### Commentary: 
@@ -19,30 +19,26 @@
 ##' @title Print IPA object
 ##' @param x Object obtained with \code{IPA}
 ##' @param digits Number of digits
-##' @param percent Logical. If \code{TRUE} show percentages.
 ##' @param ... passed to print
 ##' 
 ##' @method print IPA
 ##' @export 
 ##' @author Thomas A. Gerds <tag@@biostat.ku.dk>
-print.IPA <- function(x,percent=TRUE,digits=2,...){
+print.IPA <- function(x,digits=2,...){
     Brier=IPA.drop=IPA=NULL
     if (missing(digits)){
-        if (percent==TRUE) digits <- 1 else digits <- 3
+        digits <- 1 
     }
-    if(percent==TRUE){
-        X <- copy(x)
-        data.table::setDT(X)
-        fmt <- paste0("%1.",digits[[1]],"f")
-        X[,Brier:=sprintf(fmt=fmt,100*Brier)]
-        X[,IPA:=sprintf(fmt=fmt,100*IPA)]
-        if (match("IPA.drop",colnames(X),nomatch=0)) X[,IPA.drop:=sprintf(fmt=fmt,100*IPA.drop)]
-        print(X,...)
-        message("\nNOTE: Values are multiplied by 100 and given in % (use print(...,percent=FALSE) to avoid this.")
-    }else{
-        print(x,digits=digits,...)
-    }
-    message("NOTE: IPA.drop = IPA(Full model) - IPA.")
+    X <- copy(x)
+    data.table::setDT(X)
+    fmt <- paste0("%1.",digits[[1]],"f")
+    X[,Brier:=sprintf(fmt=fmt,100*Brier)]
+    X[,IPA:=sprintf(fmt=fmt,100*IPA)]
+    if (match("IPA.drop",colnames(X),nomatch=0)) X[,IPA.drop:=sprintf(fmt=fmt,100*IPA.drop)]
+    print(X,...)
+    message("\nNOTE: Values are multiplied by 100 and given in %.")
+    message("NOTE: The higher IPA the better.")
+    message("NOTE: IPA.drop = IPA(Full model) - IPA. The higher the drop\nthe more important is the variable for the full model.")
 }
 
 
